from enum import StrEnum

from rich import print  # noqa: A004
from rich.table import Table

from file_stats.stats import ExtensionStats


class OutputType(StrEnum):
    TABLE = "table"
    PRETTY = "pretty"


def print_table(
    file_extension_stats: dict[str, ExtensionStats],
    sorted_extensions: list[str] | None = None,
) -> None:
    table = Table(title="File Statistics")
    table.add_column("Extension", style="cyan")
    table.add_column("Count", style="magenta")
    table.add_column("Largest File Size", style="green")
    table.add_column("Total File Size", style="yellow")

    for ext in sorted_extensions or sorted(file_extension_stats.keys()):
        table.add_row(
            ext,
            f"{file_extension_stats[ext]['count']:,}",
            f"{file_extension_stats[ext]['max']:,} bytes",
            f"{file_extension_stats[ext]['total']:,} bytes",
        )

    print(table)


def print_pretty(
    file_extension_stats: dict[str, ExtensionStats],
    sorted_extensions: list[str] | None = None,
) -> None:
    for ext in sorted_extensions or sorted(file_extension_stats.keys()):
        print(f"Extension: {ext}")
        print(f"  Count: {file_extension_stats[ext]['count']:,}")
        print(f"  Largest File Size: {file_extension_stats[ext]['max']:,} bytes")
        print(f"  Total File Size: {file_extension_stats[ext]['total']:,} bytes\n")



from collections import defaultdict
from enum import StrEnum
from pathlib import Path
from typing import Literal, TypedDict

from rich import print  # noqa: A004


class SortBy(StrEnum):
    EXT = "ext"
    COUNT = "count"
    TOTAL = "total"
    MAX = "max"


class ExtensionStats(TypedDict):
    count: int
    max: int
    total: int


def gather_file_extension_stats(
    directory: Path,
) -> dict[str, ExtensionStats]:
    stats: dict[str, ExtensionStats] = defaultdict(
        lambda: {"count": 0, "max": 0, "total": 0}
    )
    for file in directory.rglob("*"):
        if not file.is_file():
            continue

        ext = file.suffix.lower() if file.suffix else "no extension"
        try:
            size = file.stat().st_size
        # Would ideally catch a more specific exception here, but the exact error is
        # unknown
        except Exception as e:  # noqa: BLE001
            print(f"[red]Warning:[/] Unable to access {file}: {e}")
            continue

        stats[ext]["count"] += 1
        stats[ext]["total"] += size
        stats[ext]["max"] = max(stats[ext]["max"], size)

    return stats


def _sort_extensions_by_key(
    file_extension_stats: dict[str, ExtensionStats],
    key: Literal["count", "total", "max"],
) -> list[str]:
    return sorted(
        file_extension_stats.keys(),
        key=lambda ext: file_extension_stats[ext][key],
        reverse=True,
    )


def get_sorted_extensions(
    file_extension_stats: dict[str, ExtensionStats], sort_by: SortBy
) -> list[str]:
    # This function looks like it could be simplified by passing the sort_by value
    # directly to the _sort_extensions_by_key function, but it's risky, as the SortBy
    # enum could be modified in the future with new values that don't correspond to the
    # keys in the ExtensionStats dict.
    if sort_by == SortBy.COUNT:
        return _sort_extensions_by_key(file_extension_stats, "count")
    if sort_by == SortBy.TOTAL:
        return _sort_extensions_by_key(file_extension_stats, "total")
    if sort_by == SortBy.MAX:
        return _sort_extensions_by_key(file_extension_stats, "max")
    return sorted(file_extension_stats.keys())



from pathlib import Path
from typing import Annotated

import typer

from file_stats.output import OutputType, print_pretty, print_table
from file_stats.stats import (
    SortBy,
    gather_file_extension_stats,
    get_sorted_extensions,
)

app = typer.Typer(rich_markup_mode="rich")


@app.command(
    help=(
        """
        [bold]Scans all files in the given directory (recursively) and collects
        statistics by file extension.[/bold]

        - [cyan]Number of files[/cyan] with each extension
        - [magenta]Size in bytes of the largest file[/magenta] with each extension
        - [green]Total file size in bytes of all files[/green] with each extension
        """
    )
)
def scan(
    directory: Annotated[
        Path | None,
        typer.Argument(
            help="Directory to scan. Defaults to current working directory.",
            exists=True,
            file_okay=False,
            dir_okay=True,
            resolve_path=True,
        ),
    ] = None,
    output: Annotated[
        OutputType,
        typer.Option("--output", "-o", help="Output format", case_sensitive=False),
    ] = OutputType.TABLE,
    sort_by: Annotated[
        SortBy,
        typer.Option(
            "--sort-by",
            "-s",
            help="Sort by field: ext, count, total, max",
            case_sensitive=False,
        ),
    ] = SortBy.EXT,
) -> None:
    """
    Scans all files in the given directory (recursively) and collects
    statistics by file extension
    """
    if directory is None:
        directory = Path.cwd()

    file_extension_stats = gather_file_extension_stats(directory)
    sorted_extensions = get_sorted_extensions(file_extension_stats, sort_by)

    if output == OutputType.TABLE:
        print_table(file_extension_stats, sorted_extensions)
    else:
        print_pretty(file_extension_stats, sorted_extensions)


if __name__ == "__main__":
    app()
